package com.atguigu.ssyx.activity.service.impl;

import com.atguigu.ssyx.activity.mapper.ActivityInfoMapper;
import com.atguigu.ssyx.activity.mapper.ActivityRuleMapper;
import com.atguigu.ssyx.activity.mapper.ActivitySkuMapper;
import com.atguigu.ssyx.activity.service.ActivityInfoService;
import com.atguigu.ssyx.client.product.ProductFeignClient;
import com.atguigu.ssyx.model.activity.ActivityInfo;
import com.atguigu.ssyx.model.activity.ActivityRule;
import com.atguigu.ssyx.model.activity.ActivitySku;
import com.atguigu.ssyx.model.product.SkuInfo;
import com.atguigu.ssyx.vo.activity.ActivityRuleVo;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Transactional
public class ActivityInfoServiceImpl extends ServiceImpl<ActivityInfoMapper, ActivityInfo> implements ActivityInfoService {
    @Autowired
    private ProductFeignClient productFeignClient;
    @Autowired
    private ActivityRuleMapper activityRuleMapper;
    @Autowired
    private ActivitySkuMapper activitySkuMapper;

    /**
     * 获取分页列表
     *
     * @param pageParam 分页查询条件
     * @return 分页结果
     */
    @Override
    public IPage<ActivityInfo> selectPage(Page<ActivityInfo> pageParam) {
        QueryWrapper<ActivityInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("id");
        IPage<ActivityInfo> page = baseMapper.selectPage(pageParam, queryWrapper);

        page.getRecords().forEach(item -> {
            item.setActivityTypeString(item.getActivityType().getComment());
        });
        return page;
    }

    /**
     * 获取活动规则
     *
     * @param id 当前id
     * @return 返回集合
     */
    @Override
    public Map<String, Object> findActivityRuleList(Long id) {
        Map<String, Object> result = new HashMap<>();

        // 活动信息列表
        List<ActivityRule> activityInfoList = activityRuleMapper.selectList(
                Wrappers.<ActivityRule>lambdaQuery().eq(ActivityRule::getActivityId, id)
        );
        result.put("activityRuleList", activityInfoList);

        // 查询活动列表
        List<ActivitySku> activitySkuList = activitySkuMapper.selectList(
                Wrappers.<ActivitySku>lambdaQuery().eq(ActivitySku::getActivityId, id)
        );
        List<Long> skuIdList = activitySkuList.stream().map(ActivitySku::getSkuId).collect(Collectors.toList());

        // 通过远程调用根据skuId得到商品信息
        List<SkuInfo> skuInfoList = productFeignClient.findSkuInfoList(skuIdList).getData();
        result.put("skuInfoList", skuInfoList);
        return result;
    }

    /**
     * 新增活动规则
     *
     * @param vo 活动规则
     */
    @Override
    public void saveActivityRule(ActivityRuleVo vo) {
        Long activityId = vo.getActivityId();
        List<ActivityRule> activityRuleList = vo.getActivityRuleList();
        List<ActivitySku> activitySkuList = vo.getActivitySkuList();

        // 根据id查询活动信息
        ActivityInfo activityInfo = getById(activityId);

        // 删除之前规则数据
        activityRuleMapper.delete(Wrappers.<ActivityRule>lambdaQuery().eq(ActivityRule::getActivityId, activityId));
        activitySkuMapper.delete(Wrappers.<ActivitySku>lambdaQuery().eq(ActivitySku::getActivityId, activityId));

        // 获取规则列表数据
        activityRuleList.forEach(activityRule -> {
            activityRule.setActivityId(activityId);
            activityRule.setActivityType(activityInfo.getActivityType());
            activityRuleMapper.insert(activityRule);
        });

        // 获取规则范围数据
        activitySkuList.forEach(activitySku -> {
            activitySku.setActivityId(activityId);
            activitySkuMapper.insert(activitySku);
        });
    }

    /**
     * 根据关键字获取sku列表，活动使用
     *
     * @param keyword 关键字
     * @return 活动列表
     */
    @Override
    public List<SkuInfo> findSkuInfoByKeyword(String keyword) {
        // 根据关键字查询sku匹配内容
        List<SkuInfo> skuInfoList = productFeignClient.findSkuInfoByKeyword(keyword).getData();

        if (!skuInfoList.isEmpty()) {
            List<Long> skuIdList = skuInfoList.stream().map(SkuInfo::getId).collect(Collectors.toList());

            // 判断添加商品之前是否参加过活动
            List<Long> existSkuIdList = baseMapper.selectSkuIdListExist(skuIdList);
            // 如果参加的话就剔除
            skuInfoList.removeIf(skuInfo -> existSkuIdList.contains(skuInfo.getId()));

            // 返回结果
            return skuInfoList;
        }
        return skuInfoList;
    }
}
